@@ -568,7 +568,18 @@ func (txn *Transaction) Commit(ctx context.Context) (commitLSN storage.LSN, retu
 		return 0, err
 	}
 
-	defer prometheus.NewTimer(txn.metrics.commitDuration(txn.write)).ObserveDuration()
+	commitTimer := prometheus.NewTimer(txn.metrics.commitDuration(txn.write))
+	defer func() {
+		duration := commitTimer.ObserveDuration()
+		// Log the commit duration to the gRPC request log
+		if customFields := logging.CustomFieldsFromContext(ctx); customFields != nil {
+			if txn.write {
+				customFields.RecordSum("transaction.commit.write_ms", int(duration.Milliseconds()))
+			} else {
+				customFields.RecordSum("transaction.commit.read_ms", int(duration.Milliseconds()))
+			}
+		}
+	}()
 
 	defer func() {
 		if err := txn.finishUnadmitted(); err != nil && returnedErr == nil {
@@ -931,6 +942,8 @@ type TransactionManager struct {
 	// left around after crashes. The files are temporary and any leftover files are expected to be cleaned up when
 	// Gitaly starts.
 	stagingDirectory string
+	// snapshotDirectory is a path to a directory where this TransactionManager should create the transaction snapshots.
+	snapshotDirectory string
 	// commandFactory is used to spawn git commands without a repository.
 	commandFactory gitcmd.CommandFactory
 	// repositoryFactory is used to build localrepo.Repo instances.
@@ -1022,6 +1035,7 @@ type transactionManagerParameters struct {
 	DB                       keyvalue.Transactioner
 	StorageName, StoragePath string
 	StateDir, StagingDir     string
+	SnapshotDir              string
 	OffloadingSink           *offloading.Sink
 	CmdFactory               gitcmd.CommandFactory
 	RepositoryFactory        localrepo.StorageScopedFactory
@@ -1056,6 +1070,7 @@ func NewTransactionManager(parameters *transactionManagerParameters) *Transactio
 		conflictMgr:         conflict.NewManager(),
 		fsHistory:           fshistory.New(),
 		stagingDirectory:    parameters.StagingDir,
+		snapshotDirectory:   parameters.SnapshotDir,
 		cleanupWorkers:      cleanupWorkers,
 		cleanupWorkerFailed: make(chan struct{}),
 		committedEntries:    list.New(),
@@ -1731,7 +1746,15 @@ func (mgr *TransactionManager) processTransaction(ctx context.Context) (returned
 	select {
 	case transaction = <-mgr.admissionQueue:
 		defer trace.StartRegion(ctx, "processTransaction").End()
-		defer prometheus.NewTimer(mgr.metrics.transactionProcessingDurationSeconds).ObserveDuration()
+
+		timer := prometheus.NewTimer(mgr.metrics.transactionProcessingDurationSeconds)
+		defer func() {
+			duration := timer.ObserveDuration()
+			// Log the transaction processing duration to the gRPC request log
+			if customFields := logging.CustomFieldsFromContext(ctx); customFields != nil {
+				customFields.RecordSum("transaction.processing_ms", int(duration.Milliseconds()))
+			}
+		}()
 
 		// The transaction does not finish itself anymore once it has been admitted for
 		// processing. This avoids the client concurrently removing the staged state
@@ -2051,11 +2074,6 @@ func (mgr *TransactionManager) CloseSnapshots() error {
 	return mgr.snapshotManager.Close()
 }
 
-// snapshotsDir returns the directory where the transactions' snapshots are stored.
-func (mgr *TransactionManager) snapshotsDir() string {
-	return filepath.Join(mgr.stagingDirectory, "snapshots")
-}
-
 // initialize initializes the TransactionManager's state from the database. It initializes WAL log manager and the
 // applied LSNs and initializes the notification channels that synchronize transaction beginning with log entry
 // applying.
@@ -2074,12 +2092,9 @@ func (mgr *TransactionManager) initialize(ctx context.Context) error {
 	if err := mgr.logManager.Initialize(ctx, mgr.appliedLSN); err != nil {
 		return fmt.Errorf("initialize log management: %w", err)
 	}
-	if err := os.Mkdir(mgr.snapshotsDir(), mode.Directory); err != nil {
-		return fmt.Errorf("create snapshot manager directory: %w", err)
-	}
 
 	var err error
-	if mgr.snapshotManager, err = snapshot.NewManager(mgr.logger, mgr.storagePath, mgr.snapshotsDir(), mgr.metrics.snapshot); err != nil {
+	if mgr.snapshotManager, err = snapshot.NewManager(mgr.logger, mgr.storagePath, mgr.snapshotDirectory, mgr.metrics.snapshot); err != nil {
 		return fmt.Errorf("new snapshot manager: %w", err)
 	}
 
